﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics;
using System.Threading;

namespace ProcessStatistics
{
    class Program
    {
        static void Main(string[] args)
        {

            Version1();
            Console.WriteLine("================================================================");
            Version2();
            Console.WriteLine("================================================================");
             Version3();
            Console.WriteLine("================================================================");
            Version4();
            Console.WriteLine("================================================================");
            Version5();

            Version6();
        }

        private static void Version1()
        {
            var processes =
                (from process in
                     Process.GetProcesses()
                 where (process.Threads.Count > 1)
                 orderby process.BasePriority descending,
                 process.PeakWorkingSet64 descending
                 select new
                        {
                            Name = process.ProcessName,
                            Priority = process.BasePriority,
                            MaxMemory = process.PeakWorkingSet64
                        }).Take(20);

            foreach (var p in processes)
                Console.WriteLine("Name: {0,25}, MaxMemory: {1,15}, Priority: {2,5}",
                    p.Name, p.MaxMemory, p.Priority);
        }
































        private static void Version2()
        {
            var processes =
                System.Diagnostics.Process.GetProcesses().
                Where(process => process.Threads.Count > 1).
                OrderByDescending(process => process.PeakWorkingSet64).
                OrderByDescending(process => process.BasePriority).Select(
                process => new
                           {
                               Name = process.ProcessName,
                               Priority = process.BasePriority,
                               MaxMemory = process.PeakWorkingSet64
                           }).Take(25);

            foreach (var p in processes)
                Console.WriteLine("Name: {0,25}, MaxMemory: {1,15}, Priority: {2,5}",
                    p.Name, p.MaxMemory, p.Priority);
        }























        private static void Version3()
        {
            //IEnumerable<ProcessStatistics2> processes =
            //    System.Diagnostics.Process.GetProcesses().
            //    Where(process => process.Threads.Count > 1).
            //    OrderByDescending(process => process.PeakWorkingSet64).
            //    OrderByDescending(process => process.BasePriority).Select(
            //    process => new ProcessStatistics2(
            //                       process.ProcessName,
            //                       process.BasePriority,
            //                       process.PeakWorkingSet64
            //               )).Take(25);

            //foreach (ProcessStatistics2 p in processes)
            //    Console.WriteLine("Name: {0,25}, MaxMemory: {1,15}, Priority: {2,5}",
            //        p.Name, p.MaxMemory, p.Priority);
        }





















        private static void Version4()
        {

            IEnumerable<ProcessStatistics> processes =
                Enumerable.Take(
                Enumerable.Select(
                Enumerable.OrderByDescending(
                Enumerable.OrderByDescending(
                Enumerable.Where(
                System.Diagnostics.Process.GetProcesses(),
                process => process.Threads.Count > 1),
                process => process.PeakWorkingSet64),
                process => process.BasePriority),
                process => new ProcessStatistics(
                           process.ProcessName,
                           process.BasePriority,
                           process.PeakWorkingSet64
                           )),
                           25);

            foreach (ProcessStatistics p in processes)
                Console.WriteLine("Name: {0,25}, MaxMemory: {1,15}, Priority: {2,5}",
                p.Name, p.MaxMemory, p.Priority);
        }



        private static void Version5()
        {
            IEnumerable<System.Diagnostics.Process> whereResult =
                Enumerable.Where(
                System.Diagnostics.Process.GetProcesses(),
                process => process.Threads.Count > 1);

            IEnumerable<System.Diagnostics.Process> secondaryOrder =
                Enumerable.OrderByDescending(
                whereResult,
                process => process.PeakWorkingSet64);

            IEnumerable<System.Diagnostics.Process> primaryOrder =
                Enumerable.OrderByDescending(
                secondaryOrder,
                process => process.BasePriority);

            IEnumerable<ProcessStatistics> stats =
                Enumerable.Select(
                primaryOrder,
                process => new ProcessStatistics(
                process.ProcessName,
                process.BasePriority,
                process.PeakWorkingSet64));

            IEnumerable<ProcessStatistics> processes =
                Enumerable.Take(
                stats,
                25);

            foreach (ProcessStatistics p in processes)
                Console.WriteLine("Name: {0,25}, MaxMemory: {1,15}, Priority: {2,5}",
                p.Name, p.MaxMemory, p.Priority);
        }





























        private static void Version6()
        {
            IEnumerable<System.Diagnostics.Process> whereResult =
                Enumerable.Where(
                System.Diagnostics.Process.GetProcesses(),
                delegate(System.Diagnostics.Process process)
                {
                    return process.Threads.Count > 1;
                }
            );

            IEnumerable<System.Diagnostics.Process> secondaryOrder =
                Enumerable.OrderByDescending(
                whereResult,
                delegate(System.Diagnostics.Process process)
                {
                    return process.PeakWorkingSet64;
                }
            );

            IEnumerable<System.Diagnostics.Process> primaryOrder =
                Enumerable.OrderByDescending(
                secondaryOrder,
                delegate(System.Diagnostics.Process process)
                {
                    return process.BasePriority;
                }
            );

            IEnumerable<ProcessStatistics> stats =
                Enumerable.Select(
                primaryOrder,
                delegate(System.Diagnostics.Process process)
                {
                    return new ProcessStatistics(
                    process.ProcessName,
                    process.BasePriority,
                    process.PeakWorkingSet64);
                }
            );

            IEnumerable<ProcessStatistics> processes =
                Enumerable.Take(
                stats,
                25);

            foreach (ProcessStatistics p in processes)
                Console.WriteLine("Name: {0,25}, MaxMemory: {1,15}, Priority: {2,5}",
                p.Name, p.MaxMemory, p.Priority);
        }
    }
}
